/*
 * File         : Choleski_h.java
 * Author       : Guansong Zhang
 * Revisor		: Qiang Zheng
 * Created      : Sat Jan 24 17:29:26 1998
 * Revision     : $Revision: 1.2 $
 * Updated      : $Date: 1998/04/21 21:14:10 $
 * Copyright: Northeast Parallel Architectures Center
 *            at Syracuse University 1998
 */

import hpjava.spmd.*;
import hpjava.spmd.adlib.*;
import hpjava.lang.*;

class Choleski_h {

  static final int n=8;

  public static void main(String argv[]) {
    SpmdEnv spmd = new SpmdEnv(argv);

    Procs1 p = new Procs1(4);

    if (p.on()) {

	  int x = 8;
      Range y = new CyclicRange(n, p.dim(0));

      Array2cdF a = new Array2cdF(p,x,y);

      int ic;
      Index jc = new Index(y);

      for (ic = 0; ic < x; ic++) {
	jc.dim().over();
	for (jc.begin(); jc.test(); jc.next()) {
	  if (y.idx(jc)>=ic)
	    a.dat()[a.pos(ic,jc)] = ic + 1; // a.data[a.element(ic,jc)]=ic.idx()+1;
	  else
	    a.dat()[a.pos(ic,jc)] = y.idx(jc)+1; // a.data[a.element(ic,jc)]=jc.idx()+1;
	} jc.dim().revo();
      }
      Adlib.rprintln("A = ");
      Adlib.aprintf("%7.3f%"+n+"N", a);

      Array1cF b = new Array1cF(p,x);

      int i;
      Location j;

      i = 0;
      j = y.location(0);

	if (j.at()) {
	  a.dat()[a.pos(i,j)] = (float) Math.sqrt(a.dat()[a.pos(i,j)]);
	} j.ta();
      
      for (int k=0; k<n-1; k++) {
	for (ic = k+1; ic < n; ic++) {
	  j = y.location(k);
	  if (j.at()) {
	    a.dat()[a.pos(ic,j)] /= a.dat()[a.pos(k,j)];
	  } j.ta();
	}

	Array1cF	bsec = new Array1cF(p,b.rng()[0].subrng(n-k-1,k+1),b.str()[0],b.dat(),b.off());
	Array1cF	asec = new Array1cF(p.divide(y.location(k)),a.rng()[0].subrng(n-k-1,k+1),a.str()[0],a.dat(),a.str()[1].offset(y.location(k)));
	Adlib.remap(bsec,asec);

	jc = new Index (y.subrng(n-k-1, k+1));
	jc.dim().over();
	for (jc.begin(); jc.test(); jc.next()) {
	  for (ic = k+1+y.idx(jc); ic < n; ic++ ) { 
		a.dat()[a.pos(ic,jc)] -= 
			b.dat()[ic] * b.dat()[b.pos(k+1+y.idx(jc))];
	  }
	} jc.dim().revo();

	i = k + 1;
	j = y.location(k+1);

	  if (j.at()) {
	    a.dat()[a.pos(i,j)]= (float) Math.sqrt(a.dat()[a.pos(i,j)]);
	  } j.ta();
  Adlib.rprintln(k+"th A = ");
  Adlib.aprintf("%7.3f%"+n+"N", a);
      }

      Adlib.rprintln("A = ");
      Adlib.aprintf("%7.3f%"+n+"N", a);
          
    } p.no();
 
    spmd.finalize();

  }
}
